/////////////////////////////////////////////////////////////////////////////
// MODULE: Splash.h
// LANGUAJE: ANSI C
// COMPILER: LCC-Win32
// OS: Windows 9x, NT
//
// AUTHOR: Daniel Guerrero Miralles (daguer@geocities.com)
// LEGAL INFORMATION: This is PUBLIC DOMAIN source code.
//
// VERSION: 2.0.0 (final) March 29th, 1999.
// ABSTRACT: Functions to display an Splash Screen on application start up.
//
// NOTES:
// This version relies on DIB library functions for drawing.
//

#ifndef _SPLASH_H_
#define _SPLASH_H_

#include <Windows.h>
#include "Dib.h"


// Initialize and run an Splash window.
// Parameters
//   hInstace:   handle of the application instance
//   hParentWnd: handle of the parent window
//   hDib:       handle of the DIB image to display in the splash
//   dwDelay:    number of milliseconds the splash will live
// Return value
//   The handle of the Splash window on success, or NULL on failure.
// NOTES:
//   - If the hParentWnd parameter is NULL the Splash will run in pop-up mode,
//   and if it is a valid window handle it will run as a child window. See
//   Splash.txt for details.
//   - If dwDelay is 0, the Splash will not be automatically destroyed, you
//   will have to expicily call Splash_Kill() to remove it from the screen.
//   - The Splash window keeps its own copy of the DIB, so you are free to
//   release it after calling to this function.

HWND Splash_Run(HINSTANCE hInstance, HWND hParentWnd, HDIB hDib, DWORD dwDelay);



// Check if the Splash is still running.
// Parameter
//   hwnd: handle of the Splash window.
// Return value
//   TRUE if the Splash window is still running (visible), or FALSE otherwise.

BOOL Splash_IsRunning(HWND hwnd);



// Hide the Splash window and destroy it.
// Parameter
//   hwnd: handle of the Splash window.
// NOTES:
//   - You can destroy an Splash window at any time with this funcion (for
//   example, if your program failed to do some initialization), but it is
//   mandatory to call it if you specified 0 as the dwDelay parameter of the
//   Splash_Run() function.

void Splash_Kill(HWND hwnd);



// Set a progress message to be displayed over the Splash image
// Paramters
//   hwnd:    handle of the Splash window.
//   pszText: pointer to the string to be diaplyed.
// NOTES:
//   - If the Splash is running on pop-up mode the string will be also
//   displayed on the Windows task bar.

void Splash_SetProgressText(HWND hwnd, LPCTSTR pszText);



// Set the color in which the Splash displays progress messages.
// Parameters
//   hwnd:    handle of the Splash window.
//   crColor: color for the text (you can use the RGB() macro to define it)
//   fRedraw: TRUE forces the Splash text to be repainted inmediately.

void Splash_SetTextColor(HWND hwnd, COLORREF crColor, BOOL fRedraw);



#endif // _SPLASH_H_
