/////////////////////////////////////////////////////////////////////////////
// MODULE: Splash.c
// LANGUAJE: ANSI C
// COMPILER: LCC-Win32
// OS: Windows 9x, NT
//
// AUTHOR: Daniel Guerrero Miralles (daguer@geocities.com)
// LEGAL INFORMATION: This is PUBLIC DOMAIN source code.
//
// VERSION: 2.0.0 (final) March 29th, 1999.
// ABSTRACT: Functions to display an Splash Screen on application start up.
// See Splash2.h to get information about function parameters.
//

//#include <Windows.h>
#include <WindowsX.h>
#include <Process.h>
#include "Splash.h"
#include "Dib.h"

// Types ////////////////////////////////////////////////////////////////////

typedef struct SPLASHINFO_TAG
{
	UINT uTimerId;
	HDIB hImage;
	COLORREF crTextColor;
	LPCTSTR pszText;
} SPLASHINFO, * PSPLASHINFO;

typedef struct SPLASHMAINARGS_TAG
{
	HINSTANCE hInst; // IN
	HDIB hImage;     // IN
	DWORD dwDelay;   // IN
	BOOL fDone;      // OUT
	HWND hwnd;       // OUT
} SPLASHMAINARGS, * PSPLASHMAINARGS;

typedef struct SPLASHCREATEPARAMS_TAG
{
	HDIB hImage;
	DWORD dwDelay;
} SPLASHCREATEPARAMS, * PSPLASHCREATEPARAMS;

// Constants and macros /////////////////////////////////////////////////////

#define SPLASH_TIMER_EVENT 1
#define MSG_KILLSPLASH (WM_USER + 0)
#define MSG_SETCOLOR    (WM_USER + 1)

#define GetSplashInfo(hw) ((PSPLASHINFO)GetWindowLong(hw, GWL_USERDATA))
#define SetSplashInfo(hw,psi) (SetWindowLong(hw,GWL_USERDATA,(LONG)psi))

#define RECT_WIDTH(rc) ((rc).right - (rc).left)
#define RECT_HEIGHT(rc) ((rc).bottom - (rc).top)

// Message handlers /////////////////////////////////////////////////////////

static void Splash_MsgPaint(HWND hwnd)
{
	PSPLASHINFO pSplashInfo;
	HDC hDC, hMemDC;
	RECT rcWndRect;
	HBITMAP hBitmap, hOldBitmap;
	HPALETTE hDibPalette, hDCPalette;//, hMemDCPalette;

	pSplashInfo = GetSplashInfo(hwnd);
	hDC = GetDC(hwnd);
	hMemDC = CreateCompatibleDC(hDC);
	hBitmap = CreateCompatibleBitmap(hDC, DIB_Width(pSplashInfo->hImage)
		, DIB_Height(pSplashInfo->hImage));
	hOldBitmap = SelectObject(hMemDC, hBitmap);

	GetClientRect(hwnd, &rcWndRect);

	if (DIB_HasPalette(pSplashInfo->hImage))
	{
		hDibPalette = DIB_GetPalette(pSplashInfo->hImage);
		hDCPalette = SelectPalette(hDC, hDibPalette, FALSE);
//		hMemDCPalette = SelectPalette(hMemDC, hDibPalette, FALSE);
		RealizePalette(hDC);
		RealizePalette(hMemDC);
	}

	DIB_Paint(pSplashInfo->hImage, hMemDC, 0, 0);

	if (pSplashInfo->pszText != NULL)
	{
		SetBkMode(hMemDC, TRANSPARENT);
		SetTextColor(hMemDC, pSplashInfo->crTextColor);
		DrawText(hMemDC, pSplashInfo->pszText, -1, &rcWndRect
			, DT_CENTER|DT_BOTTOM|DT_SINGLELINE);
	}

	BitBlt(hDC, 0, 0, DIB_Width(pSplashInfo->hImage)
		, DIB_Height(pSplashInfo->hImage), hMemDC, 0, 0, SRCCOPY);

	if (DIB_HasPalette(pSplashInfo->hImage))
	{
		SelectPalette(hDC, hDCPalette, FALSE);
		SelectPalette(hMemDC, hMemDC, FALSE);
		DeleteObject(hDibPalette);
	}

	SelectObject(hMemDC, hOldBitmap);
	DeleteObject(hBitmap);
	DeleteDC (hMemDC);
	ReleaseDC (hwnd, hDC);

	FORWARD_WM_PAINT(hwnd, DefWindowProc);
}

static void Splash_PaletteChange(HWND hwnd)
{
	HDC hDC;
	HPALETTE hDibPalette, hDCPalette;
	PSPLASHINFO pSplashInfo;

	pSplashInfo = GetSplashInfo(hwnd);

	hDC = GetDC(hwnd);
	hDibPalette = DIB_GetPalette(pSplashInfo->hImage);
	hDCPalette = SelectPalette (hDC, hDibPalette, FALSE);
	if (RealizePalette(hDC) > 0)
	{
		RedrawWindow(hwnd, NULL, NULL, RDW_INTERNALPAINT);
	}
	SelectPalette(hDC, hDCPalette, FALSE);
	DeleteObject(hDibPalette);
	ReleaseDC(hwnd, hDC);
}

static void Splash_MsgPaletteChanged(HWND hwnd, HWND hwFocusWnd)
{
	if (hwnd != hwFocusWnd)
	{
		Splash_PaletteChange(hwnd);
	}
}

static BOOL Splash_MsgQueryNewPalette(HWND hwnd)
{
	Splash_PaletteChange(hwnd);
	return TRUE;
}

static void Splash_MsgTimer(HWND hwnd, UINT uEventId)
{
//	PSPLASHINFO pSplashInfo;

//	pSplashInfo = GetSplashInfo(hwnd);
	if (uEventId == SPLASH_TIMER_EVENT)
	{
		DestroyWindow(hwnd);
	}
}

static void Splash_MsgSetText(HWND hwnd, LPCTSTR pszText)
{
	PSPLASHINFO pSplashInfo;

	pSplashInfo = GetSplashInfo(hwnd);
	if (pSplashInfo->pszText != NULL)
	{
		GlobalFree((HGLOBAL)pSplashInfo->pszText);
	}

	pSplashInfo->pszText = GlobalAlloc(GPTR, lstrlen(pszText) + 1);
	if (pSplashInfo->pszText != NULL)
	{
		lstrcpy(pSplashInfo->pszText, pszText);
	}

	RedrawWindow(hwnd, NULL, NULL, RDW_INTERNALPAINT);
	FORWARD_WM_SETTEXT(hwnd,pszText,DefWindowProc);
}

static BOOL Splash_MsgCreate(HWND hwnd, LPCREATESTRUCT pCS)
{
	PSPLASHCREATEPARAMS pCreateParams;
	PSPLASHINFO pSplashInfo;

	pCreateParams = (PSPLASHCREATEPARAMS) pCS->lpCreateParams;
	pSplashInfo = GlobalAlloc(GPTR, sizeof(SPLASHINFO));
	if (pSplashInfo == NULL)
		return FALSE;

	pSplashInfo->pszText = NULL;
	pSplashInfo->crTextColor = RGB(0,0,0);

	pSplashInfo->hImage = DIB_Duplicate(pCreateParams->hImage);
	if (pSplashInfo->hImage == (HDIB) NULL)
	{
		GlobalFree(pSplashInfo);
		return FALSE;
	}

	if (pCreateParams->dwDelay > 0)
	{
		pSplashInfo->uTimerId = SetTimer (hwnd, SPLASH_TIMER_EVENT
			, pCreateParams->dwDelay, NULL);
		if (pSplashInfo->uTimerId == 0)
		{
			DIB_Release(pSplashInfo->hImage);
			GlobalFree(pSplashInfo);
			return FALSE;
		}
	}

	SetLastError(0);
	SetSplashInfo(hwnd, pSplashInfo);
	return (GetLastError() == 0);
}

static void Splash_MsgDestroy(HWND hwnd)
{
	PSPLASHINFO pSplashInfo;

	pSplashInfo = GetSplashInfo(hwnd);
	SetSplashInfo(hwnd,NULL);
	if (pSplashInfo->pszText != NULL)
	{
		GlobalFree((HGLOBAL)pSplashInfo->pszText);
	}
	if (pSplashInfo->uTimerId != 0)
	{
		KillTimer(hwnd, pSplashInfo->uTimerId);
	}
	DIB_Release(pSplashInfo->hImage);
	GlobalFree((HGLOBAL)pSplashInfo);
	if (GetParent(hwnd) == NULL)
	{
		PostQuitMessage(0);
	}
}

void Splash_MsgSetColor(HWND hwnd, COLORREF crColor, BOOL fRedraw)
{
	PSPLASHINFO pSplashInfo;

	pSplashInfo = GetSplashInfo(hwnd);
	pSplashInfo->crTextColor = crColor;
	if (fRedraw)
	{
		RedrawWindow(hwnd, NULL, NULL, RDW_INTERNALPAINT);
	}
}

// Callback functions ///////////////////////////////////////////////////////

static LRESULT CALLBACK Splash_WndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		HANDLE_MSG(hwnd, WM_CREATE, Splash_MsgCreate);
		HANDLE_MSG(hwnd, WM_PAINT, Splash_MsgPaint);
		HANDLE_MSG(hwnd, WM_PALETTECHANGED, Splash_MsgPaletteChanged);
		HANDLE_MSG(hwnd, WM_QUERYNEWPALETTE, Splash_MsgQueryNewPalette);
		HANDLE_MSG(hwnd, WM_TIMER, Splash_MsgTimer);
		HANDLE_MSG(hwnd, WM_SETTEXT, Splash_MsgSetText);
		HANDLE_MSG(hwnd, WM_DESTROY, Splash_MsgDestroy);
		case MSG_KILLSPLASH:
			DestroyWindow(hwnd);
			return 0;
		case MSG_SETCOLOR:
			Splash_MsgSetColor(hwnd, (COLORREF)wParam, (BOOL)lParam);
			return 0;
	}
	return DefWindowProc(hwnd, uMsg, wParam, lParam);
}

// Internal helpers /////////////////////////////////////////////////////////

static HWND Splash_CreateAsPopup(HINSTANCE hInstance, HDIB hImage, DWORD dwDelay)
{
	SPLASHCREATEPARAMS CreateParams;

	CreateParams.hImage = hImage;
	CreateParams.dwDelay = dwDelay;

	return CreateWindowEx(WS_EX_TOPMOST|WS_EX_DLGMODALFRAME
		, "SplashScreenClass", NULL, WS_POPUP|WS_DISABLED, 0, 0
		, DIB_Width(hImage), DIB_Height(hImage), NULL, NULL, hInstance
		, (PVOID)&CreateParams);
}

static HWND Splash_CreateAsChild(HINSTANCE hInstance, HWND hwParent, HDIB hImage, DWORD dwDelay)
{
	SPLASHCREATEPARAMS CreateParams;

	CreateParams.hImage = hImage;
	CreateParams.dwDelay = dwDelay;

	return CreateWindowEx(WS_EX_TOPMOST|WS_EX_DLGMODALFRAME
		, "SplashScreenClass", NULL, WS_CHILD|WS_DISABLED, 0, 0
		, DIB_Width(hImage), DIB_Height(hImage), hwParent, NULL, hInstance
		, (PVOID)&CreateParams);
}

static BOOL Splash_RegisterClass(HINSTANCE hInstance)
{
	WNDCLASS wc;

	if (!GetClassInfo(hInstance, "SplashScreenClass", &wc))
	{
		wc.style = 0;
		wc.lpfnWndProc = Splash_WndProc;
		wc.cbClsExtra = 0;
		wc.cbWndExtra = sizeof(PSPLASHINFO);
		wc.hInstance = hInstance;
		wc.hIcon = NULL;
		wc.hCursor = NULL;
		wc.hbrBackground = (HBRUSH)GetStockObject(NULL_BRUSH);
		wc.lpszMenuName = NULL;
		wc.lpszClassName = "SplashScreenClass";
		return (RegisterClass(&wc) != 0);
	}

	return TRUE;
}

static void CenterWindow (HWND hwnd)
{
	RECT rcWndRect;
	RECT rcParentRect;
	int x, y, nScrWidth, nScrHeight;
	HWND hwParent;

	hwParent = GetParent(hwnd);
	if (hwParent == NULL)
	{
		nScrWidth = GetSystemMetrics(SM_CXSCREEN);
		nScrHeight = GetSystemMetrics(SM_CYSCREEN);
	}
	else
	{
		GetClientRect(hwParent, &rcParentRect);
		nScrWidth = RECT_WIDTH(rcParentRect);
		nScrHeight = RECT_HEIGHT(rcParentRect);
	}

	GetWindowRect(hwnd, &rcWndRect);

	x = (nScrWidth - RECT_WIDTH(rcWndRect)) / 2;
	y = (nScrHeight - RECT_HEIGHT(rcWndRect)) / 2;

	SetWindowPos(hwnd, NULL, x, y, 0, 0, SWP_NOSIZE|SWP_NOZORDER);
}

// This is the pop-up mode thread main function
static void Splash_Main(PVOID pVoid)
{
	PSPLASHMAINARGS pArgs = (PSPLASHMAINARGS) pVoid;
	HWND hwnd;
	MSG msg;
	BOOL fGotMsg;

	hwnd = Splash_CreateAsPopup(pArgs->hInst, pArgs->hImage, pArgs->dwDelay);
	if (hwnd == (HWND)NULL)
	{
		pArgs->hwnd = (HWND)NULL;
		pArgs->fDone = TRUE;
		_endthread();
	}

	CenterWindow(hwnd);
	ShowWindow(hwnd, SW_SHOW);

	pArgs->hwnd = hwnd;
	pArgs->fDone = TRUE;

	// From that point we cannot access pArgs, that may have been deleted

	fGotMsg = GetMessage(&msg, NULL, 0, 0);
	while (fGotMsg)
	{
		if (fGotMsg == -1)
		{
			// Critical error
			break;
		}
		DispatchMessage(&msg);
		fGotMsg = GetMessage(&msg, NULL, 0, 0);
	}

	_endthread();
}

// Interface functions //////////////////////////////////////////////////////

HWND Splash_Run(HINSTANCE hInstance, HWND hParentWnd, HDIB hDib, DWORD dwDelay)
{
	SPLASHMAINARGS SplashMainArgs;
	HWND hwnd;

	if (!Splash_RegisterClass(hInstance))
	{
		return (HWND)NULL;
	}

	// If hParentWnd is null we need to create the Splash as a pop-up
	// window, with its own message loop in a separate thread.
	if (hParentWnd == NULL)
	{
		SplashMainArgs.hInst = hInstance;
		SplashMainArgs.hImage = hDib;
		SplashMainArgs.dwDelay = dwDelay;
		SplashMainArgs.fDone = FALSE;
		SplashMainArgs.hwnd = (HWND)NULL;
		_beginthread(Splash_Main, 2000, (PVOID)&SplashMainArgs);
		while (!SplashMainArgs.fDone)
		{
			// Wait for the thread to set up the window handle
		}
		return SplashMainArgs.hwnd;
	}
	// If it is going to be a child window, simply create it and
	// return the window handle
	hwnd = Splash_CreateAsChild(hInstance, hParentWnd, hDib, dwDelay);
	if (hwnd != (HWND) NULL)
	{
		CenterWindow(hwnd);
		ShowWindow(hwnd, SW_SHOW);
	}
	return hwnd;
}

BOOL Splash_IsRunning(HWND hwnd)
{
	return IsWindow(hwnd);
}

void Splash_Kill(HWND hwnd)
{
	if (Splash_IsRunning(hwnd))
		PostMessage(hwnd, MSG_KILLSPLASH, 0, 0);
}

void Splash_SetProgressText(HWND hwnd, LPCTSTR pszText)
{
	if (Splash_IsRunning(hwnd))
		FORWARD_WM_SETTEXT(hwnd,pszText,SendMessage);
}

void Splash_SetTextColor(HWND hwnd, COLORREF crColor, BOOL fRedraw)
{
	if (Splash_IsRunning(hwnd))
		SendMessage(hwnd, MSG_SETCOLOR, (WPARAM)crColor, (LPARAM)fRedraw);
}
